package cn.com.poc.knowledge.aggregate.impl;


import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.service.BosConfigService;
import cn.com.poc.common.utils.*;
import cn.com.poc.knowledge.aggregate.KnowledgeService;
import cn.com.poc.knowledge.constant.KnowledgeConstant;
import cn.com.poc.knowledge.entity.BizKnowledgeDocumentEntity;
import cn.com.poc.knowledge.entity.BizMemberKnowledgeRelationEntity;
import cn.com.poc.knowledge.query.KnowledgeQueryItem;
import cn.com.poc.knowledge.service.BizKnowledgeDocumentService;
import cn.com.poc.knowledge.service.BizMemberKnowledgeRelationService;
import cn.com.poc.message.entity.TrainKnowledgeMessage;
import cn.com.poc.message.service.KnowledgeProducerService;
import cn.com.poc.support.security.oauth.entity.UserBaseEntity;
import cn.com.yict.framemax.core.exception.BusinessException;
import cn.com.yict.framemax.data.model.PagingInfo;
import cn.hutool.core.bean.BeanUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class KnowledgeServiceImpl implements KnowledgeService {


    @Resource
    private BizKnowledgeDocumentService bizKnowledgeDocumentService;

    @Resource
    private BizMemberKnowledgeRelationService bizMemberKnowledgeRelationService;

    @Resource
    private KnowledgeProducerService knowledgeProducerService;


    @Resource
    private BosConfigService bosConfigService;

    @Override
    public List<BizKnowledgeDocumentEntity> uploadDocument(MultipartFile[] documentFiles) throws Exception {
        //获取用户类型
        UserBaseEntity userBaseEntity = BlContext.getCurrentUserNotException();
        String memberId = userBaseEntity.getUserId().toString();
        String memberType = "system";

        String trainStatus = KnowledgeConstant.TrainStatus.UNOPENED;//默认状态-未执行
        Date uploadTime = new Date();//上传时间
        //记录成功
        List<BizKnowledgeDocumentEntity> result = new ArrayList<>();
        for (MultipartFile documentFile : documentFiles) {
            //获取文件名
            String documentName = documentFile.getOriginalFilename();

            // 查看是否有重名
            List<KnowledgeQueryItem> knowledgeQueryItems = bizKnowledgeDocumentService.searchKnowledge(null, null, memberId, documentName, null);
            if (CollectionUtils.isNotEmpty(knowledgeQueryItems)) {
                throw new BusinessException("上传文件存在重名,文件名:" + documentName);
            }

            // 统计文件字符数
            String type = documentFile.getOriginalFilename().substring(documentFile.getOriginalFilename().lastIndexOf(".") + 1, documentFile.getOriginalFilename().length());
            File file = File.createTempFile(UUIDTool.getUUID(), "." + type);
            documentFile.transferTo(file);
            // 文件大小不能超过10M
            long fileSizeInBytes = file.length();
            double fileSizeInMB = (double) fileSizeInBytes / (1024 * 1024);
            if (fileSizeInMB > 5) {
                throw new BusinessException("上传的文件不能超过5M,文件名:" + documentName);
            }

            String fileContent = DocumentLoad.documentToText(file).replaceAll(StringUtils.LF, StringUtils.EMPTY).replaceAll(StringUtils.CR, StringUtils.EMPTY);//文件内容
            if (StringUtils.isBlank(fileContent)) {
                throw new BusinessException("错误的文件，文件内容不能为空,文件名: " + documentName);
            }
            //获取文件字符数
            long charCount = fileContent.length();
            //文件字符数不能超过100w
            if (charCount > 2 * 10000) {
                throw new BusinessException("文件内容字符数不能超过2w,文件名: " + documentName);
            }

            //文件上传
            String documentUrl = bosConfigService.upload(Files.newInputStream(file.toPath()),type, documentFile.getContentType());

            //保存记录
            BizKnowledgeDocumentEntity bizKnowledgeDocumentEntity = new BizKnowledgeDocumentEntity();
            bizKnowledgeDocumentEntity.setOwner(KnowledgeConstant.Owner.USER);
            bizKnowledgeDocumentEntity.setKnowledgeType(KnowledgeConstant.KnowledgeType.BASE);
            bizKnowledgeDocumentEntity.setDocumentName(documentName);
            bizKnowledgeDocumentEntity.setDocumentUrl(documentUrl);
            bizKnowledgeDocumentEntity.setCharCount(charCount);
            bizKnowledgeDocumentEntity.setUploadTime(uploadTime);
            bizKnowledgeDocumentEntity.setIsEnable(CommonConstant.YOrN.N);
            bizKnowledgeDocumentEntity.setKnowledgeId(null);
            bizKnowledgeDocumentEntity.setTrainStatus(trainStatus);
            bizKnowledgeDocumentEntity.setIsDeleted(CommonConstant.IsDeleted.N);
            BizKnowledgeDocumentEntity knowledgeDocumentEntity = bizKnowledgeDocumentService.save(bizKnowledgeDocumentEntity);

            //保存关联
            BizMemberKnowledgeRelationEntity bizMemberKnowledgeRelationEntity = new BizMemberKnowledgeRelationEntity();
            bizMemberKnowledgeRelationEntity.setMemberId(memberId);
            bizMemberKnowledgeRelationEntity.setMemberType(memberType);
            bizMemberKnowledgeRelationEntity.setKdId(knowledgeDocumentEntity.getKdId());
            bizMemberKnowledgeRelationEntity.setIsDeleted(CommonConstant.YOrN.N);
            bizMemberKnowledgeRelationService.save(bizMemberKnowledgeRelationEntity);

            //构造返回
            result.add(knowledgeDocumentEntity);
        }
        return result;
    }

    @Override
    public Boolean delDocument(Integer kdId) {
        Assert.notNull(kdId, "kdId cannot be null");
        bizKnowledgeDocumentService.deleted(kdId);
        bizMemberKnowledgeRelationService.deletedByKdId(kdId);
        return true;
    }

    @Override
    public Boolean renameDocument(Integer kdId, String rename) throws Exception {
        Assert.notNull(kdId);
        Assert.notBlank(rename);
        BizKnowledgeDocumentEntity bizKnowledgeDocumentEntity = bizKnowledgeDocumentService.get(kdId);
        if (bizKnowledgeDocumentEntity == null) {
            throw new BusinessException("document not exist");
        }
        bizKnowledgeDocumentEntity.setDocumentName(rename);
        bizKnowledgeDocumentService.update(kdId, bizKnowledgeDocumentEntity);
        return true;
    }

    @Override
    public String enableKnowledge(Integer kdId) throws Exception {
        BizKnowledgeDocumentEntity bizKnowledgeDocumentEntity = bizKnowledgeDocumentService.get(kdId);

        String isEnable = bizKnowledgeDocumentEntity.getIsEnable();
        if (CommonConstant.YOrN.N.equals(isEnable)) {
            bizKnowledgeDocumentEntity.setIsEnable(CommonConstant.YOrN.Y);
        } else {
            bizKnowledgeDocumentEntity.setIsEnable(CommonConstant.YOrN.N);
        }
        BizKnowledgeDocumentEntity documentEntity = bizKnowledgeDocumentService.update(kdId, bizKnowledgeDocumentEntity);
        return documentEntity.getIsEnable();
    }

    @Override
    public Boolean trainKnowledge(List<Integer> kdIds) throws Exception {
        Assert.notEmpty(kdIds);
        Assert.isTrue(kdIds.size() <= 5, "Training cannot exceed five documents");
        for (Integer kdId : kdIds) {
            BizKnowledgeDocumentEntity bizKnowledgeDocumentEntity = bizKnowledgeDocumentService.get(kdId);
            // 扣减次数
//            AiGcMemberEntity currentMember = aiGcVipService.getCurrentMember(BlContext.getCurrentUserNotException());
//            Map<Integer, Integer> reduced = bizAiGcMemberFreqService.reduceNumber(currentMember.getRelationId(), AiGcPaymentConfigurationConstant.DOMAIN.TRAIN, currentMember.getType(), bizKnowledgeDocumentEntity.getCharCount().intValue());

            String currentStatus = bizKnowledgeDocumentEntity.getTrainStatus();
            if (KnowledgeConstant.TrainStatus.COMPLETE.equals(currentStatus) ||
                    KnowledgeConstant.TrainStatus.LINE.equals(currentStatus) ||
                    KnowledgeConstant.TrainStatus.TRAINING.equals(currentStatus)) {
                throw new BusinessException("No repetitive training");
            }

            bizKnowledgeDocumentEntity.setTrainStatus(KnowledgeConstant.TrainStatus.LINE);
            bizKnowledgeDocumentService.update(kdId, bizKnowledgeDocumentEntity);

            TrainKnowledgeMessage message = new TrainKnowledgeMessage();
            message.setKid(kdId);
            message.setFileUrl(bizKnowledgeDocumentEntity.getDocumentUrl());
//            message.setReduced(reduced);
            knowledgeProducerService.trainKnowledge(message);

        }
        return true;
    }

    @Override
    public List<BizKnowledgeDocumentEntity> getList(String search, String trainStatus, PagingInfo pagingInfo) throws Exception {
        List<BizKnowledgeDocumentEntity> result = new ArrayList<>();
        String memberId = BlContext.getCurrentUser().getUserId().toString();
        if (StringUtils.isBlank(memberId)) {
            memberId = BlContext.getCurrentUserNotException().getUserId().toString();
        }
        List<KnowledgeQueryItem> knowledgeQueryItems = bizKnowledgeDocumentService.searchKnowledge(search, trainStatus, memberId, null, pagingInfo);
        if (CollectionUtils.isNotEmpty(knowledgeQueryItems)) {
            result = knowledgeQueryItems.stream().map(item -> {
                BizKnowledgeDocumentEntity entity = new BizKnowledgeDocumentEntity();
                BeanUtil.copyProperties(item, entity);
                return entity;
            }).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public List<BizKnowledgeDocumentEntity> getListByKdIds(List<Integer> kdIds) {
        Assert.notEmpty(kdIds);
        List<BizKnowledgeDocumentEntity> res = new ArrayList<>();
        for (Integer kdId : kdIds) {
            BizKnowledgeDocumentEntity bizKnowledgeDocumentEntity = bizKnowledgeDocumentService.get(kdId);
            res.add(bizKnowledgeDocumentEntity);
        }
        return res;
    }
}
